@@ -14,11 +14,17 @@ import os
 import time
 import uuid
 
+from cinderclient.v2 import client as cinderclient
 import fixtures
-from keystoneclient.v2_0 import client as keystoneclient
+from glanceclient import client as glanceclient
+from keystoneauth1.exceptions import discovery as discovery_exc
+from keystoneauth1 import identity
+from keystoneauth1 import session as ksession
+from keystoneclient import client as keystoneclient
+from keystoneclient import discover as keystone_discover
 import os_client_config
 import six
-import tempest_lib.cli.base
+import tempest.lib.cli.base
 import testtools
 
 import novaclient
@@ -30,23 +36,43 @@ BOOT_IS_COMPLETE = ("login as 'cirros' user. default password: "
                     "'cubswin:)'. use 'sudo' for root.")
 
 
+def is_keystone_version_available(session, version):
+    """Given a (major, minor) pair, check if the API version is enabled."""
+
+    d = keystone_discover.Discover(session)
+    try:
+        d.create_client(version)
+    except (discovery_exc.DiscoveryFailure, discovery_exc.VersionNotAvailable):
+        return False
+    else:
+        return True
+
+
 # The following are simple filter functions that filter our available
 # image / flavor list so that they can be used in standard testing.
 def pick_flavor(flavors):
     """Given a flavor list pick a reasonable one."""
-    for flavor in flavors:
-        if flavor.name == 'm1.tiny':
-            return flavor
-    for flavor in flavors:
-        if flavor.name == 'm1.small':
-            return flavor
+    for flavor_priority in ('m1.nano', 'm1.micro', 'm1.tiny', 'm1.small'):
+        for flavor in flavors:
+            if flavor.name == flavor_priority:
+                return flavor
     raise NoFlavorException()
 
 
 def pick_image(images):
+    firstImage = None
     for image in images:
-        if image.name.startswith('cirros') and image.name.endswith('-uec'):
+        firstImage = firstImage or image
+        if image.name.startswith('cirros') and (
+                image.name.endswith('-uec') or
+                image.name.endswith('-disk.img')):
             return image
+
+    # We didn't find the specific cirros image we'd like to use, so just use
+    # the first available.
+    if firstImage:
+        return firstImage
+
     raise NoImageException()
 
 
@@ -80,6 +106,9 @@ class NoCloudConfigException(Exception):
     pass
 
 
+CACHE = {}
+
+
 class ClientTestBase(testtools.TestCase):
     """Base test class for read only python-novaclient commands.
 
@@ -167,26 +196,48 @@ class ClientTestBase(testtools.TestCase):
         passwd = auth_info['password']
         tenant = auth_info['project_name']
         auth_url = auth_info['auth_url']
-        if 'insecure' in cloud_config.config:
-            insecure = cloud_config.config['insecure']
-        else:
-            insecure = False
+        user_domain_id = auth_info['user_domain_id']
+        self.project_domain_id = auth_info['project_domain_id']
 
-        if self.COMPUTE_API_VERSION == "2.latest":
-            version = novaclient.API_MAX_VERSION.get_string()
+        if 'insecure' in cloud_config.config:
+            self.insecure = cloud_config.config['insecure']
         else:
-            version = self.COMPUTE_API_VERSION or "2"
-        self.client = novaclient.client.Client(
-            version, user, passwd, tenant,
-            auth_url=auth_url, insecure=insecure)
+            self.insecure = False
+
+        auth = identity.Password(username=user,
+                                 password=passwd,
+                                 project_name=tenant,
+                                 auth_url=auth_url,
+                                 project_domain_id=self.project_domain_id,
+                                 user_domain_id=user_domain_id)
+        session = ksession.Session(auth=auth, verify=(not self.insecure))
+
+        self.client = self._get_novaclient(session)
+
+        self.glance = glanceclient.Client('2', session=session)
 
         # pick some reasonable flavor / image combo
-        self.flavor = pick_flavor(self.client.flavors.list())
-        self.image = pick_image(self.client.images.list())
-        self.network = pick_network(self.client.networks.list())
+        if "flavor" not in CACHE:
+            CACHE["flavor"] = pick_flavor(self.client.flavors.list())
+        if "image" not in CACHE:
+            CACHE["image"] = pick_image(self.glance.images.list())
+        self.flavor = CACHE["flavor"]
+        self.image = CACHE["image"]
+
+        if "network" not in CACHE:
+            tested_api_version = self.client.api_version
+            proxy_api_version = novaclient.api_versions.APIVersion('2.35')
+            if tested_api_version > proxy_api_version:
+                self.client.api_version = proxy_api_version
+            try:
+                # TODO(mriedem): Get the networks from neutron if using neutron
+                CACHE["network"] = pick_network(self.client.networks.list())
+            finally:
+                self.client.api_version = tested_api_version
+        self.network = CACHE["network"]
 
         # create a CLI client in case we'd like to do CLI
-        # testing. tempest_lib does this really weird thing where it
+        # testing. tempest.lib does this really weird thing where it
         # builds a giant factory of all the CLIs that it knows
         # about. Eventually that should really be unwound into
         # something more sensible.
@@ -194,18 +245,68 @@ class ClientTestBase(testtools.TestCase):
             'OS_NOVACLIENT_EXEC_DIR',
             os.path.join(os.path.abspath('.'), '.tox/functional/bin'))
 
-        self.cli_clients = tempest_lib.cli.base.CLIClient(
+        self.cli_clients = tempest.lib.cli.base.CLIClient(
             username=user,
             password=passwd,
             tenant_name=tenant,
             uri=auth_url,
             cli_dir=cli_dir,
-            insecure=insecure)
+            insecure=self.insecure)
 
-        self.keystone = keystoneclient.Client(username=user,
-                                              password=passwd,
-                                              tenant_name=tenant,
-                                              auth_url=auth_url)
+        self.keystone = keystoneclient.Client(session=session,
+                                              username=user,
+                                              password=passwd)
+        self.cinder = cinderclient.Client(auth=auth, session=session)
+
+        if "use_neutron" not in CACHE:
+            # check to see if we're running with neutron or not
+            for service in self.keystone.services.list():
+                if service.type == 'network':
+                    CACHE["use_neutron"] = True
+                    break
+            else:
+                CACHE["use_neutron"] = False
+
+    def _get_novaclient(self, session):
+        nc = novaclient.client.Client("2", session=session)
+
+        if self.COMPUTE_API_VERSION:
+            if "min_api_version" not in CACHE:
+                # Obtain supported versions by API side
+                v = nc.versions.get_current()
+                if not hasattr(v, 'version') or not v.version:
+                    # API doesn't support microversions
+                    CACHE["min_api_version"] = (
+                        novaclient.api_versions.APIVersion("2.0"))
+                    CACHE["max_api_version"] = (
+                        novaclient.api_versions.APIVersion("2.0"))
+                else:
+                    CACHE["min_api_version"] = (
+                        novaclient.api_versions.APIVersion(v.min_version))
+                    CACHE["max_api_version"] = (
+                        novaclient.api_versions.APIVersion(v.version))
+
+            if self.COMPUTE_API_VERSION == "2.latest":
+                requested_version = min(novaclient.API_MAX_VERSION,
+                                        CACHE["max_api_version"])
+            else:
+                requested_version = novaclient.api_versions.APIVersion(
+                    self.COMPUTE_API_VERSION)
+
+            if not requested_version.matches(CACHE["min_api_version"],
+                                             CACHE["max_api_version"]):
+                msg = ("%s is not supported by Nova-API. Supported version" %
+                       self.COMPUTE_API_VERSION)
+                if CACHE["min_api_version"] == CACHE["max_api_version"]:
+                    msg += ": %s" % CACHE["min_api_version"].get_string()
+                else:
+                    msg += "s: %s - %s" % (
+                        CACHE["min_api_version"].get_string(),
+                        CACHE["max_api_version"].get_string())
+                self.skipTest(msg)
+
+            nc.api_version = requested_version
+        return nc
 
     def nova(self, action, flags='', params='', fail_ok=False,
              endpoint_type='publicURL', merge_stderr=False):
@@ -218,14 +319,14 @@ class ClientTestBase(testtools.TestCase):
                                poll_interval=1):
         """Wait until volume reaches given status.
 
-        :param volume_id: uuid4 id of given volume
+        :param volume: volume resource
         :param status: expected status of volume
         :param timeout: timeout in seconds
         :param poll_interval: poll interval in seconds
         """
         start_time = time.time()
         while time.time() - start_time < timeout:
-            volume = self.client.volumes.get(volume.id)
+            volume = self.cinder.volumes.get(volume.id)
             if volume.status == status:
                 break
             time.sleep(poll_interval)
@@ -233,7 +334,7 @@ class ClientTestBase(testtools.TestCase):
             self.fail("Volume %s did not reach status %s after %d s"
                       % (volume.id, status, timeout))
 
-    def wait_for_server_os_boot(self, server_id, timeout=60,
+    def wait_for_server_os_boot(self, server_id, timeout=300,
                                 poll_interval=1):
         """Wait until instance's operating system  is completely booted.
 
@@ -242,13 +343,15 @@ class ClientTestBase(testtools.TestCase):
         :param poll_interval: poll interval in seconds
         """
         start_time = time.time()
+        console = None
         while time.time() - start_time < timeout:
-            if BOOT_IS_COMPLETE in self.nova('console-log %s ' % server_id):
+            console = self.nova('console-log %s ' % server_id)
+            if BOOT_IS_COMPLETE in console:
                 break
             time.sleep(poll_interval)
         else:
-            self.fail("Server %s did not boot after %d s"
-                      % (server_id, timeout))
+            self.fail("Server %s did not boot after %d s.\nConsole:\n%s"
+                      % (server_id, timeout, console))
 
     def wait_for_resource_delete(self, resource, manager,
                                  timeout=60, poll_interval=1):
@@ -332,7 +435,39 @@ class ClientTestBase(testtools.TestCase):
                     # the value now using the column index.
                     return line.split("|")[1:-1][column_index].strip()
 
-        raise ValueError("Unable to find value for column '%s'.")
+        raise ValueError("Unable to find value for column '%s'." % column)
+
+    def _get_list_of_values_from_single_column_table(self, table, column):
+        """Get the list of values for the column in the single-column table
+
+        Example table:
+
+        +------+
+        | Tags |
+        +------+
+        | tag1 |
+        | tag2 |
+        +------+
+
+        :param table: newline-separated table with |-separated cells
+        :param column: name of the column to look for
+        :raises: ValueError if the single column has some other name
+        """
+        lines = table.split("\n")
+        column_name = None
+        values = []
+        for line in lines:
+            if "|" in line:
+                if not column_name:
+                    column_name = line.split("|")[1].strip()
+                    if column_name != column:
+                        raise ValueError(
+                            "The table has no column %(expected)s "
+                            "but has column %(actual)s." % {
+                                'expected': column, 'actual': column_name})
+                else:
+                    values.append(line.split("|")[1].strip())
+        return values
 
     def _create_server(self, name=None, with_network=True, add_cleanup=True,
                        **kwargs):
@@ -350,6 +485,18 @@ class ClientTestBase(testtools.TestCase):
             'building', ['active'])
         return server
 
+    def _get_project_id(self, name):
+        """Obtain project id by project name."""
+        if self.keystone.version == "v3":
+            project = self.keystone.projects.find(name=name)
+        else:
+            project = self.keystone.tenants.find(name=name)
+        return project.id
+
+    def skip_if_neutron(self):
+        if CACHE["use_neutron"]:
+            self.skipTest('nova-network is not available')
+
 
 class TenantTestBase(ClientTestBase):
     """Base test class for additional tenant and user creation which
@@ -358,21 +505,41 @@ class TenantTestBase(ClientTestBase):
 
     def setUp(self):
         super(TenantTestBase, self).setUp()
-        tenant = self.keystone.tenants.create(
-            self.name_generate('v' + self.COMPUTE_API_VERSION))
-        self.tenant_id = tenant.id
-        self.addCleanup(self.keystone.tenants.delete, self.tenant_id)
         user_name = self.name_generate('v' + self.COMPUTE_API_VERSION)
+        project_name = self.name_generate('v' + self.COMPUTE_API_VERSION)
         password = 'password'
-        self.user_id = self.keystone.users.create(
-            user_name, password, tenant_id=self.tenant_id).id
+
+        if self.keystone.version == "v3":
+            project = self.keystone.projects.create(project_name,
+                                                    self.project_domain_id)
+            self.project_id = project.id
+            self.addCleanup(self.keystone.projects.delete, self.project_id)
+
+            self.user_id = self.keystone.users.create(
+                name=user_name, password=password,
+                default_project=self.project_id).id
+
+            for role in self.keystone.roles.list():
+                if "member" in role.name.lower():
+                    self.keystone.roles.grant(role.id, user=self.user_id,
+                                              project=self.project_id)
+                    break
+        else:
+            project = self.keystone.tenants.create(project_name)
+            self.project_id = project.id
+            self.addCleanup(self.keystone.tenants.delete, self.project_id)
+
+            self.user_id = self.keystone.users.create(
+                user_name, password, tenant_id=self.project_id).id
+
         self.addCleanup(self.keystone.users.delete, self.user_id)
-        self.cli_clients_2 = tempest_lib.cli.base.CLIClient(
+        self.cli_clients_2 = tempest.lib.cli.base.CLIClient(
             username=user_name,
             password=password,
-            tenant_name=tenant.name,
+            tenant_name=project_name,
             uri=self.cli_clients.uri,
-            cli_dir=self.cli_clients.cli_dir)
+            cli_dir=self.cli_clients.cli_dir,
+            insecure=self.insecure)
 
     def another_nova(self, action, flags='', params='', fail_ok=False,
                      endpoint_type='publicURL', merge_stderr=False):
